/* this file contains useful text macros, which save work during
   programming */
#ifndef FEINS_MACROS_H
#define FEINS_MACROS_H

#include <stdio.h>
#include <stdlib.h>

/* still in development ==> debug switch on */
#define DEBUGFEINS /* */

/* define the integer datatypes, */
/* FIDX is for indices into arrays/vectors, also general integer type */
#define FIDX long 

/* new adjoint handling, without replacing rows in the matrices */
#define NEW_ADJOINT

/* type of coarse grid stabilisation:
   0=none,
   1=lower Re:    use lower Re on coarser grids
   2=crude SUPG:  2nd order elements, leave out all terms that vanish
                  in 1st order elements case, only add stab term for
                  advection 
   3=lin ixSUPG:  use linear elements in the coarse meshes, only add
                  stab term for advection
   4=lin SUPG AD: use only advection diffusion matrices (for linear
                  elements) on coarser meshes
   5=art diff:    define diffusion coefficient locally in each cell
                  such that stability is ensured  
   6=full SUPG:   all terms dictated by the subgrid idea of Hughes */
#define STABTYPE 2

#define RAMAGE_NOT_TEZDUYAR  /* use the stabilisation parameter
                                used by Ramage rather than that by Tezduyar */

/*#define TIMETERM /* include an artificial time term into the solution
		    process, hopefully improving performance of the
		    linear solvers at the cost of slight slowing down
		    of the nonlinear solve */

#define FP_VELOCITY_SOLVER 1 /* the type of velocity solver used in
				the F_p preconditioner, 
				0 - GMRES, no preco
				1 - GMRES, V-cycle_MG (uses
				    MG_FOURDIR_SWEEPS (if so,
				    MG_SMOOTHER_DAMPING as well), all
				    STABTYPEs, MG_MIN_EN) 
				2 - GMRES, W-cycle_MG (uses MG_W_ARMS,
				    MG_W_DAMPING, STABTYPES 0,1,2,5,
				    USE_SORTED_GS)
			     */
/* #define MG_FOUR_DIR_SWEEPS /* do sorted Gauss-Seidel Sweeps,
			      left-right, bottom-top, right-left,
			      top-bottom */
/* #define MG_W_DAMPING  /* use damped coarse grid updates */
#define MG_W_ARMS 2   /* number of CGCs in each W-cycle-level
			 (1=V-cycle, 2=W-cycle,...) */
#define MG_SMOOTHER_DAMPING 1.0 /* damping in GS sweeps */
/* #define USE_SORTED_GS /* use sorted Gauss-Seidel sweeps in the
			 smoother (rather than unsorted) */
/* #define MG_MIN_EN /* switch for minimal energy MG step */

/* #define DIAG_ONLY_COARSE
/* */
	
/* #define MG_F_NO_CMAT     /* switch OFF exact coarse-grid solves */		 
/* #define MG_F_NO_DIAGPROJ /* switch OFF projection of the diagonals in
			    Gauss-Seidel Smoother */		 

/* switch for conservative formulation of the advective term (only in
   Newton type linearisation) */
/*#define CONSERV_ADV_NEWT
/* */


/* use a coarse grid solver in the multigrid solver for the pressure
   space Laplacian in the F_p preconditioner,
   !!! still problems with "no boundary conditions", therefore deactivated !!!
*/
/* #define LAPLACIAN_COARSE_GRID_SOLVER
/* */

/* test wave resolution on the mesh (for the
   advection-diffusion-reaction problems that arise from the Newton
   linearization of the Navier-Stokes system) */
/* #define WAVE_RESOLVE_TEST
/* */




/***********************************************************************
 **                                                                   **
 **                                                                   **
 **   SDFEM/Box-Smoother options                                      **
 **                                                                   **
 **                                                                   **
 ***********************************************************************/
/*#define BOXSMOOTHER_SORTED          /* switch on sorting of the sweeps
                                       of local problems, sort by x,y */
/*#define BOXSMOOTHER_SORTED_2DIR     /* switch on a second sorted
				       sweep, sorted by -y,-x */
/*#define BOXSMOOTHER_ELEMBASED       /* switch on element-based smoother,
                                       otherwise p-node based */
/*#define BOXSMOOTHER_LOC_NEUMAN      /* switch to Neumann-like local
				       problems, in p-node based,
				       otherwise local Dirichlet */
/*#define BOXSMOOTHER_DIAG_ONLY_KLOC  /* use only diagonal of K_loc
				       velocity block, otherwise full
				       K_loc  */ 
#define BOXSMOOTHER_DAMPING (0.5)   /* damping factor for the smoother */
/* #include "box_smoother_damping.h" /* */
#define BOXSMOOTHER_BC_ELEMENT_ALL_V /*use all v-dofs for boundary
				       elements, otherwise as defined
				       by switch local
				       Dirichlet/Neumann problems */
#define BOXSMOOTHER_VEL_WEIGHTS     /* Weight the velocity dofs in the
				       local problems by
				       1/sqrt(#appearances) */





/* return value convention */
#define SUCCESS 0
#define FAIL    1

/* maximal refinement dept */
#define FEINSLEVELMAX 30

/* macro for malloc failure returns */
#define TRY_MALLOC(__who__, __somuch__, __type__, __func__) {		\
    __who__ = (__type__*) malloc( (__somuch__)*sizeof(__type__) );	\
    if ((__who__)==NULL)						\
      {									\
	fprintf(stderr,							\
		"line-%d in %s, file %s: malloc for %d %s entries to %s failed\n",\
		__LINE__, #__func__, __FILE__, (int)(__somuch__), #__type__, #__who__ );	\
	return FAIL;							\
      }									\
  }

#ifdef __cplusplus
#define TRY_NEW(__who__, __somuch__, __type__, __func__) {		\
    __who__ = new __type__[__somuch__] ;				\
    if ((__who__)==NULL)						\
      {									\
	fprintf(stderr,							\
		"line-%d in %s, file %s: new for %d %s entries to %s failed\n",\
		__LINE__, #__func__, __FILE__, (int)(__somuch__), #__type__, #__who__ );	\
	return FAIL;							\
      }									\
  }
#endif


/* macro for malloc failure returns */
#define TRY_REALLOC(__who__, __somuch__, __type__, __func__) {		\
    __who__ = (__type__*) realloc( (__who__),				\
				   (__somuch__)*sizeof(__type__) );	\
    if ((__who__)==NULL)						\
      {									\
	fprintf(stderr,							\
		"line-%d in %s, file %s: realloc for %d %s entries to %s failed\n",	\
		__LINE__, #__func__, __FILE__, (int)(__somuch__), #__type__, #__who__ ); \
	return FAIL;							\
      }									\
  }

/* macro to handle error returns by subroutines, 
   __where__ = function in which this happens
   __who__   = the subroutine called
   __err__   = variable holding the return value of the subroutine */
#define FUNCTION_FAILURE_HANDLE( __err__, __who__, __where__) {\
 if ((__err__)!=SUCCESS) \
   {\
      fprintf(stderr, \
      "line-%d in %s, file %s: call to %s returned with %d(!=SUCCESS)\n",\
	      __LINE__, #__where__, __FILE__, #__who__, __err__ );\
      return FAIL; \
   }\
  }

/* too much trouble with fmax => define it ourself */
#define fmax(Val1, Val2)  (Val1>Val2)? Val1:Val2
#define fmin(Val1, Val2)  (Val1<Val2)? Val1:Val2

#endif
